#include "enemy.h"


extern SDL_Surface *screen;
extern SPRITE *player;
extern double fps_factor;

void enemy_add_wave(int type)
{
	switch(type) {
		case SP_EN_XEV:
			enemy_xev_add();
			break;

		case SP_EN_CRUSHER:
			enemy_crusher_add();
			break;
				
		case SP_EN_EYEFO:
			enemy_eyefo_add();
			break;

		case SP_EN_CUBE:
			enemy_cube_add();
			break;

		case SP_EN_DRAGER:
			enemy_drager_add();
			break;

		case SP_EN_MINE:
			enemy_mine_add();
			break;

		case SP_EN_RWINGX:
			enemy_rwingx_add();
			break;

		case SP_EN_CIR:
			enemy_cir_add();
			break;

		case SP_EN_ZATAK:
			enemy_zatak_add();
			break;

		case SP_EN_BOSS01:
			enemy_boss01_add();
			break;
	}
}


void enemy_bullet_create(SPRITE *s, int speed)
{
	SPRITE *h;
	BULLET_DATA *data;

	h=sprite_add_file("data/kugel.bmp",1,1);	
	h->type=SP_EN_BULLET;
	h->flags|=(SP_FLAG_VISIBLE|SP_FLAG_COLCHECK);
	h->mover=enemy_bullet_move;
	h->aktframe=0;
	h->x=s->x;
	h->y=s->y;

	if((data=malloc(sizeof(BULLET_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	h->data=data;

	data->angle=atan2(player->y-s->y,player->x-s->x);
	data->speed=speed;

}

void enemy_bullet_move(SPRITE *s)
{
	BULLET_DATA *d=(BULLET_DATA *)s->data;

	s->x+=cos(d->angle)*d->speed*fps_factor;
	s->y+=sin(d->angle)*d->speed*fps_factor;
	if((s->x<0)||(s->x>640)||(s->y<0)||(s->y>480)) {
		s->type=-1;
	}
}

void enemy_homing_create(SPRITE *s)
{
	SPRITE *r;
	HOMING_DATA *data;

	r=sprite_add_file("data/rotating_rocket.bmp",24,1);
	r->flags|=(SP_FLAG_VISIBLE|SP_FLAG_COLCHECK);
	r->type=SP_EN_HOMING;
	r->x=s->x;
	r->y=s->y;
	r->mover=enemy_homing_move;
	if((data=malloc(sizeof(HOMING_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	r->data=data;
	data->b.score=5;
	data->b.health=1;
	data->angle=0;
	data->speed=2;
	data->range=200;
	data->delay=10;

	enemy_homing_update(r);
}

void enemy_homing_update(SPRITE *s)
{

	HOMING_DATA *d=(HOMING_DATA *)s->data;
	
	/* Zielwinkel erfassen */
	d->angle=(double)(((atan2(player->y-s->y,player->x-s->x)+M_PI)*180)/M_PI);
	s->aktframe=(23-(d->angle/15)+6)%24;
}

void enemy_homing_move(SPRITE *s)
{
	double dummy;
	SDL_Surface *tmp;
	HOMING_DATA *d=(HOMING_DATA *)s->data;

	d->delay-=fps_factor;
	if(d->delay >0) {
		dummy=((double)d->angle/180*M_PI)-M_PI;
		s->x+=(double)(cos(dummy)*(double)d->speed)*fps_factor;
		s->y+=(double)(sin(dummy)*(double)d->speed)*fps_factor;
	} else {
		/* nur jeden 10ten Frame Winkel aktualisieren */
		d->delay=10;
		enemy_homing_update(s);
	}


	/* max. reichweite erreicht? self-destruct! */
	d->range-=fps_factor;
	if(d->range<=0) {
		s->type=-1;
		tmp=sprite_getcurrimg(s);
		parsys_add(tmp,1,1,s->x,s->y,10,0,0,30,EXPLODE|DIFFSIZE,NULL);
		SDL_FreeSurface(tmp);
	}
}
