#include "player.h"

extern SDL_Surface *screen;
extern int keyboard[];
extern int score;
extern int lives;
extern KEYCONFIG keyconfig;
extern double fps_factor;
SPRITE *player;

void player_init()
{

	PLAYER_DATA *data;

	player=sprite_add_file("data/ship-med.bmp",11,1);
	player->type=SP_PLAYER;
	player->flags|=(SP_FLAG_VISIBLE|SP_FLAG_COLCHECK);
	player->mover=player_move;
	player->x=screen->w/2;
	player->y=screen->h-player->h;
	player->aktframe=5;
	if((data=malloc(sizeof(PLAYER_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	player->data=data;
	data->state=PL_SAVE;
	data->firewait=0;
	data->explode=0;
	data->save_delay=120;
	data->anim_delay=0;
	data->weapon=0;
	data->player_speed=3;
}

void player_move(SPRITE *s)
{

	PLAYER_DATA *d=(PLAYER_DATA *)s->data;

	switch(d->state) {

		case PL_NORMAL:

			player_keycontrol(s);
			player_colcheck(s,SP_SHOW_ENEMYS|SP_SHOW_ENEMY_WEAPONS|SP_SHOW_BONUS);

			break;

		case PL_EXPLODE:

			if(!d->explode) {
				s->flags|=SP_FLAG_VISIBLE;
				d->save_delay=120;
				d->state=PL_SAVE;
			}
			break;

		case PL_SAVE:

			player_keycontrol(s);
			player_colcheck(s,SP_SHOW_BONUS);


			if(d->save_delay) {
				d->save_delay--;
				s->alpha=128;
			} else {
				d->state=PL_NORMAL;
				s->alpha=255;
			}
			break;	
	}
		
}

void player_keycontrol(SPRITE *s)
{
	int direction=0;
	PLAYER_DATA *d=(PLAYER_DATA *)s->data;

	/* if player is invisible (state gameover) allow no keycontrol */
	if(!(s->flags&SP_FLAG_VISIBLE))
		return;
	
	if(keyboard[keyconfig.l]) {
		direction=-1;
		if(s->x>0) {
			s->x-=d->player_speed*fps_factor;
		}
	}

	if(keyboard[keyconfig.r]) {
		direction=1;
		if(s->x<screen->w-s->w) {
			s->x+=d->player_speed*fps_factor;
		}
	}

	if((keyboard[keyconfig.u]) && (s->y>0)) {
		s->y-=d->player_speed*fps_factor;
		parsys_add(NULL,50,0,s->x+s->w/2,s->y+s->h,10,270,10,100,PIXELATE,NULL);

	}

	if((keyboard[keyconfig.d]) && (s->y<screen->h-s->h)) {
		s->y+=d->player_speed*fps_factor;
	}

	if(d->firewait) d->firewait--;
	if(keyboard[keyconfig.f] && (!d->firewait)) {
		switch(d->weapon) {
			case 0:
				player_add_plasma(s);
				break;
			case 1:
				player_add_doubleplasma(s);
				break;
			case 2:
				player_add_quadplasma(s);
				break;
			case 3:
				player_add_killray(s);
				break;
			case 4:
				player_add_plasma(s);
				player_add_killray(s);
				break;
			case 5:
				player_add_doubleplasma(s);
				player_add_killray(s);
				break;
			case 6:
				player_add_quadplasma(s);
				player_add_killray(s);
				break;
		}
		d->firewait=10;
	}

	if(d->anim_delay>0) {
		d->anim_delay--;
	} else {
		d->anim_delay=3;
		switch(direction) {
			case -1:
				if(s->aktframe>0) s->aktframe--;
				break;
			case 1:
				if(s->aktframe<10) s->aktframe++;
				break;
			default:
				if(s->aktframe>5) s->aktframe--;
				if(s->aktframe<5) s->aktframe++;
				break;
		}
	}
}

void player_colcheck(SPRITE *s, int mask)
{
	SPRITE *c;
	SDL_Surface *spimg;
	PLAYER_DATA *d=(PLAYER_DATA *)s->data;

	/* Kollision Player <> Feind, Feindwaffe oder Bonusitem */
	if((c=sprite_colcheck(s,mask))!=NULL) {
		
		switch(c->type) {
			case SP_BONUS_FIREPOWER:
				if(d->weapon<6) {
					d->weapon++;
				}
				spimg=sprite_getcurrimg(c);
				parsys_add(spimg, c->w,1, c->x,c->y, 5, 0, 0, 80, LINESPLIT, NULL);
				SDL_FreeSurface(spimg);
				bonus_info_add(c->x,c->y,"data/weapon.bmp");
				c->type=-1;
				break;

			case SP_BONUS_BOMB:
				player_bomb_add(c);
				spimg=sprite_getcurrimg(c);
				parsys_add(spimg, c->w,1, c->x,c->y, 5, 0, 0, 80, LINESPLIT, NULL);
				SDL_FreeSurface(spimg);
				bonus_info_add(c->x,c->y,"data/firebomb.bmp");
				c->type=-1;
				break;

			case SP_BONUS_COIN:
				score+=1000;
				spimg=sprite_getcurrimg(c);
				parsys_add(spimg, c->w,1, c->x,c->y, 5, 0, 0, 80, LINESPLIT, NULL);
				SDL_FreeSurface(spimg);
				bonus_info_add(c->x,c->y,"data/plus1000.bmp");
				c->type=-1;
				break;

			case SP_BONUS_ADDSPEED:
				if(d->player_speed<6)
					d->player_speed++;
				spimg=sprite_getcurrimg(c);
				parsys_add(spimg, c->w,1, c->x,c->y, 5, 0, 0, 80, LINESPLIT, NULL);
				SDL_FreeSurface(spimg);
				bonus_info_add(c->x,c->y,"data/speed.bmp");
				c->type=-1;
				break;

			case SP_EN_BOSS01:
				enemy_boss01_hitbyplayer(c);
				break;

			default:	
				spimg=sprite_getcurrimg(c);
				parsys_add(spimg, 2,2, c->x,c->y, 10, 90, 10, 30, EXPLODE|DIFFSIZE, NULL);
				SDL_FreeSurface(spimg);
				spimg=NULL;
				d->state=PL_EXPLODE;
				d->explode=0;

				spimg=sprite_getcurrimg(s);
				parsys_add(spimg, 2,2, c->x,c->y, 10, 270, 10, 30, EXPLODE|DIFFSIZE, &d->explode);
				SDL_FreeSurface(spimg);
				spimg=NULL;
				c->type=-1;
				s->flags&=~SP_FLAG_VISIBLE;
				lives--;
		}
	}
}

void player_add_plasma(SPRITE *player)
{
	SPRITE *shot;
	PLASMA_DATA *data;

	shot=sprite_add_file("data/plasma.bmp",1,1);
	shot->type=SP_PL_BULLET;
	shot->x=player->x+player->w/2-shot->w/2;
	shot->y=player->y-shot->h+20;
	shot->mover=player_plasma_move;
	shot->flags|=SP_FLAG_VISIBLE;
	if((data=malloc(sizeof(PLASMA_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	shot->data=data;
	data->angle=90;
	data->speed=12;
}

void player_add_doubleplasma(SPRITE *player)
{
	SPRITE *shot;
	PLASMA_DATA *data;

	shot=sprite_add_file("data/plasma.bmp",1,1);
	shot->type=SP_PL_BULLET;
	shot->x=player->x+player->w/2-shot->w/2-5;
	shot->y=player->y-shot->h+20;
	shot->mover=player_plasma_move;
	shot->flags|=SP_FLAG_VISIBLE;
	if((data=malloc(sizeof(PLASMA_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	shot->data=data;
	data->angle=90;
	data->speed=12;

	shot=sprite_add_file("data/plasma.bmp",1,1);
	shot->type=SP_PL_BULLET;
	shot->x=player->x+player->w/2-shot->w/2+5;
	shot->y=player->y-shot->h+20;
	shot->mover=player_plasma_move;
	shot->flags|=SP_FLAG_VISIBLE;
	if((data=malloc(sizeof(PLASMA_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	shot->data=data;
	data->angle=90;
	data->speed=12;
}

void player_add_quadplasma(SPRITE *player)
{
	SPRITE *shot;
	PLASMA_DATA *data;

	shot=sprite_add_file("data/plasma.bmp",1,1);
	shot->type=SP_PL_BULLET;
	shot->x=player->x+player->w/2-shot->w/2-15;
	shot->y=player->y-shot->h+25;
	shot->mover=player_plasma_move;
	shot->flags|=SP_FLAG_VISIBLE;
	if((data=malloc(sizeof(PLASMA_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	shot->data=data;
	data->angle=90-5;
	data->speed=12;

	shot=sprite_add_file("data/plasma.bmp",1,1);
	shot->type=SP_PL_BULLET;
	shot->x=player->x+player->w/2-shot->w/2-5;
	shot->y=player->y-shot->h+20;
	shot->mover=player_plasma_move;
	shot->flags|=SP_FLAG_VISIBLE;
	if((data=malloc(sizeof(PLASMA_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	shot->data=data;
	data->angle=90;
	data->speed=12;

	shot=sprite_add_file("data/plasma.bmp",1,1);
	shot->type=SP_PL_BULLET;
	shot->x=player->x+player->w/2-shot->w/2+5;
	shot->y=player->y-shot->h+20;
	shot->mover=player_plasma_move;
	shot->flags|=SP_FLAG_VISIBLE;
	if((data=malloc(sizeof(PLASMA_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	shot->data=data;
	data->angle=90;
	data->speed=12;

	shot=sprite_add_file("data/plasma.bmp",1,1);
	shot->type=SP_PL_BULLET;
	shot->x=player->x+player->w/2-shot->w/2+15;
	shot->y=player->y-shot->h+25;
	shot->mover=player_plasma_move;
	shot->flags|=SP_FLAG_VISIBLE;
	if((data=malloc(sizeof(PLASMA_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	shot->data=data;
	data->angle=90+5;
	data->speed=12;
}

void player_add_killray(SPRITE *player)
{
	SPRITE *shot;
	KILLRAY_DATA *data;
	static int last=0;

	if(last) {
		shot=sprite_add_file("data/killray-r.bmp",12,1);
		last=0;
	} else {
		shot=sprite_add_file("data/killray-b.bmp",12,1);
		last=1;
	}
	shot->type=SP_PL_BULLET;
	shot->x=player->x+player->w/2-shot->w/2;
	shot->y=player->y-shot->h+20;
	shot->mover=player_killray_move;
	shot->flags|=SP_FLAG_VISIBLE;
	if((data=malloc(sizeof(KILLRAY_DATA)))==NULL) {
		CHECKPOINT;
		error(ERR_FATAL,"out of memory");
	}
	shot->data=data;
	data->angle=90;
	data->speed=5;
	data->dist=0;
}

void player_killray_move(SPRITE *s)
{
	double dummy;
	KILLRAY_DATA *p=(KILLRAY_DATA *)s->data;


	dummy=((double)p->angle/180*M_PI)-M_PI;
	s->x+=cos(dummy)*p->speed*fps_factor;
	s->y+=sin(dummy)*p->speed*fps_factor;
	p->dist+=p->speed*fps_factor;

	if((p->dist>20)&&(s->aktframe<11)) {
		p->dist=0;
		s->aktframe++;
	}

	if(s->y<-10) {
		s->type=-1;
	} else {
		weapon_colcheck(s,p->angle,0);
	}
}

void player_plasma_move(SPRITE *s)
{
	double dummy;
	PLASMA_DATA *p=(PLASMA_DATA *)s->data;


	dummy=((double)p->angle/180*M_PI)-M_PI;
	s->x+=cos(dummy)*p->speed*fps_factor;
	s->y+=sin(dummy)*p->speed*fps_factor;

	if(s->y<-10) {
		s->type=-1;
	} else {
		weapon_colcheck(s,p->angle,1);
	}
}

void player_bomb_add(SPRITE *s)
{
	SPRITE *c;
	BOMB_DATA *b;

	int a;

	for(a=0;a<355;a+=5) {
		c=sprite_add_file("data/fireball.bmp",1,1);
		c->type=SP_PL_BULLET;
		c->x=player->x+player->w/2;
		c->y=player->y+player->h/2;
		c->mover=player_bomb_move;
		c->flags|=SP_FLAG_VISIBLE;
		if((b=malloc(sizeof(BOMB_DATA)))==NULL) {
			CHECKPOINT;
			error(ERR_FATAL,"out of memory");
		}
		c->data=b;
		b->angle=a;
		b->speed=3;
	}
}

void player_bomb_move(SPRITE *s)
{
	BOMB_DATA *b=(BOMB_DATA *)s->data;
	double dummy;

	dummy=((double)b->angle/180*M_PI)-M_PI;
	s->x+=cos(dummy)*b->speed*fps_factor;
	s->y+=sin(dummy)*b->speed*fps_factor;

	if((s->x<0)||(s->x>640)||(s->y<0)||(s->y>480)) {
		s->type=-1;
	} else {
		weapon_colcheck(s,b->angle,1);
	}
}

void weapon_colcheck(SPRITE *s, int angle, int remove)
{
	SPRITE *c;
	SDL_Surface *spimg;
	ENEMY_BASE *d;

	if((c=sprite_colcheck(s,SP_SHOW_ENEMYS))!=NULL) {
		
		if(remove)
			s->type=-1;

		switch(c->type) {
			case SP_EN_BOSS01:
				enemy_boss01_hitbyweapon(c,angle);
				break;

			default:
				spimg=sprite_getcurrimg(c);
				parsys_add(spimg, 2,2, c->x,c->y, 10, angle, 10, 30, EXPLODE|DIFFSIZE, NULL);
				SDL_FreeSurface(spimg);

				d=(ENEMY_BASE *)c->data;
				d->health--;
				if(!d->health) {
					score+=d->score;
					c->type=-1;
				}
				break;
		}
	}
}
